clear
clc

% ==============================================================================

YoungModulus(1) = 200e9;

Area(1) = 0.5e-3;
Area(2) = 0.4e-3;

DistributedLoadQ1 = 10e3;
DistributedLoadQ2 = 10e3;

ForceT1 = 40e3;
ForceT2 = 5e3;

BoundaryConditions = [1];

% ==============================================================================

Nodes = [
  1  0    0
  2  0.4  0
  3  1.2  0
];

Elements = [
  1  1  2  1  1
  2  2  3  2  1
];

% ==============================================================================

DofPerNode = 1;

NumberOfNodes = size(Nodes, 1);
NumberOfElements = size(Elements, 1);
NumberOfDof = NumberOfNodes * DofPerNode;

% ==============================================================================

[Xe1 Ye1] = ElementCoordinates(1, Nodes, Elements);
[Xe2 Ye2] = ElementCoordinates(2, Nodes, Elements);

% ==============================================================================

Le1 = ElementLength(Xe1, Ye1);
Le2 = ElementLength(Xe2, Ye2);

% ==============================================================================

Ke1 = StiffnessMatrix(Le1, Area(Elements(1, 4)), YoungModulus(Elements(1, 5)));
Ke2 = StiffnessMatrix(Le2, Area(Elements(2, 4)), YoungModulus(Elements(2, 5)));

disp('=== Element stiffness matrices ===========================================')

disp('Ke1 = ')
disp(Ke1)

disp('Ke2 = ')
disp(Ke2)

% ==============================================================================

K = zeros(NumberOfDof);
K = MatrixAssembly(Elements(1, 2:3), DofPerNode, Ke1, K);
K = MatrixAssembly(Elements(2, 2:3), DofPerNode, Ke2, K);

disp('=== Structural stiffness matrix ==========================================')

disp('K = ')
disp(K)

% ==============================================================================

Pe1 = LoadVector(Le1, DistributedLoadQ1);
Pe2 = LoadVector(Le2, DistributedLoadQ2);

disp('=== Element load vectors =================================================')

disp('Pe1 = ')
disp(Pe1)

disp('Pe2 = ')
disp(Pe2)

% ==============================================================================

P = zeros(NumberOfDof, 1);
P = VectorAssembly(Elements(1, 2:3), DofPerNode, Pe1, P);
P = VectorAssembly(Elements(2, 2:3), DofPerNode, Pe2, P);

disp('=== Structural load vector (distributed load) ============================')

disp('P = ')
disp(P)

% ==============================================================================

F = zeros(NumberOfDof, 1);
F(2) = ForceT1;
F(3) = ForceT2;

disp('=== Structural load vector (forces) ======================================')

disp('F = ')
disp(F)

% ==============================================================================

[Q, R] = SolveEquations(K, P + F, BoundaryConditions);

disp('=== Structural displacement vector =======================================')

disp('Q = ')
disp(Q)

disp('=== Structural reaction vector ===========================================')

disp('R = ')
disp(R)

% ==============================================================================

Qe1 = ExtractDof(Elements(1, 2:3), DofPerNode, Q);
Qe2 = ExtractDof(Elements(2, 2:3), DofPerNode, Q);

disp('=== Elements displacement vectors ========================================')

disp('Qe1 = ')
disp(Qe1)

disp('Qe2 = ')
disp(Qe2)

% ==============================================================================

Re1 = ExtractDof(Elements(1, 2:3), DofPerNode, R);
Re2 = ExtractDof(Elements(2, 2:3), DofPerNode, R);

disp('=== Elements reaction vectors ============================================')

disp('Re1 = ')
disp(Re1)

disp('Re2 = ')
disp(Re2)

% ==============================================================================

Pt = 10;
x1 = [0 : Le1 / (Pt - 1) : Le1];
x2 = [0 : Le2 / (Pt - 1) : Le2];

% ==============================================================================

for i = 1 : Pt

  u1(i) = ShapeFunctions(x1(i), Le1) * Qe1;
  u2(i) = ShapeFunctions(x2(i), Le2) * Qe2;

end

% ==============================================================================

h1 = figure(1);
clf
set(h1, 'Name', 'Displacement')

subplot(2, 2, 1)
plot(x1, u1)
title('Element 1')
xlabel('x1')
ylabel('u1')

subplot(2, 2, 2)
plot(x2, u2)
title('Element 2')
xlabel('x2')
ylabel('u2')

subplot(2, 2, 3:4)
plot(x1, u1, Le1 + x2, u2)
title('Structure')
xlabel('x')
ylabel('u')

% ==============================================================================

for i = 1 : Pt

  s1(i) = YoungModulus(Elements(1, 5)) * ShapeFunctionsDiff(x1(i), Le1) * Qe1;
  s2(i) = YoungModulus(Elements(1, 5)) * ShapeFunctionsDiff(x2(i), Le2) * Qe2;

end

% ==============================================================================

h2 = figure(2);
clf
set(h2, 'Name', 'Stress')

subplot(2, 2, 1)
plot(x1, s1)
title('Element 1')
xlabel('x1')
ylabel('s1')

subplot(2, 2, 2)
plot(x2, s2)
title('Element 2')
xlabel('x2')
ylabel('s2')

subplot(2, 2, 3:4)
plot(x1, s1, Le1 + x2, s2)
title('Structure')
xlabel('x')
ylabel('s')

% ==============================================================================

for i = 1 : Pt

  v1(i) = YoungModulus(Elements(1, 5)) * Area(Elements(1, 4)) * ShapeFunctionsDiff(x1(i), Le1) * Qe1;
  v2(i) = YoungModulus(Elements(2, 5)) * Area(Elements(2, 4)) * ShapeFunctionsDiff(x2(i), Le2) * Qe2;

end

% ==============================================================================

h3 = figure(3);
clf
set(h3, 'Name', 'Normal force')

subplot(2, 2, 1)
plot(x1, v1)
title('Element 1')
xlabel('x1')
ylabel('v1')

subplot(2, 2, 2)
plot(x2, v2)
title('Element 2')
xlabel('x2')
ylabel('v2')

subplot(2, 2, 3:4)
plot(x1, v1, Le1 + x2, v2)
title('Structure')
xlabel('x')
ylabel('v')
